@@ -239,6 +239,8 @@ index((expects_dialect), 1, prolog_dialect, dialect).
 index((exists_source), 1, prolog_dialect, dialect).
 index((source_exports), 2, prolog_dialect, dialect).
 index((dif), 2, dif, dif).
+index((doc_save), 2, pldoc_files, doc_files).
+index((doc_pack), 1, pldoc_files, doc_files).
 index((doc_server), 1, pldoc_http, doc_http).
 index((doc_server), 2, pldoc_http, doc_http).
 index((doc_browser), 0, pldoc_http, doc_http).
@@ -494,6 +496,7 @@ index((set_breakpoint), 4, prolog_breakpoints, prolog_breakpoints).
 index((delete_breakpoint), 1, prolog_breakpoints, prolog_breakpoints).
 index((breakpoint_property), 2, prolog_breakpoints, prolog_breakpoints).
 index((clause_info), 4, prolog_clause, prolog_clause).
+index((initialization_layout), 4, prolog_clause, prolog_clause).
 index((predicate_name), 2, prolog_clause, prolog_clause).
 index((clause_name), 2, prolog_clause, prolog_clause).
 index((prolog_walk_code), 1, prolog_codewalk, prolog_codewalk).
@@ -502,6 +505,8 @@ index((prolog_colourise_term), 4, prolog_colour, prolog_colour).
 index((syntax_colour), 2, prolog_colour, prolog_colour).
 index((syntax_message), 3, prolog_colour, prolog_colour).
 index((prolog_history), 1, prolog_history, prolog_history).
+index((infer_meta_predicate), 2, prolog_metainference, prolog_metainference).
+index((inferred_meta_predicate), 2, prolog_metainference, prolog_metainference).
 index((pack_list_installed), 0, prolog_pack, prolog_pack).
 index((pack_info), 1, prolog_pack, prolog_pack).
 index((pack_list), 1, prolog_pack, prolog_pack).
@@ -512,6 +517,7 @@ index((pack_upgrade), 1, prolog_pack, prolog_pack).
 index((pack_rebuild), 1, prolog_pack, prolog_pack).
 index((pack_rebuild), 0, prolog_pack, prolog_pack).
 index((pack_remove), 1, prolog_pack, prolog_pack).
+index((pack_property), 2, prolog_pack, prolog_pack).
 index((prolog_server), 2, prolog_server, prolog_server).
 index((prolog_read_source_term), 4, prolog_source, prolog_source).
 index((read_source_term_at_location), 3, prolog_source, prolog_source).
@@ -521,6 +527,7 @@ index((prolog_canonical_source), 2, prolog_source, prolog_source).
 index((file_name_on_path), 2, prolog_source, prolog_source).
 index((file_alias_path), 2, prolog_source, prolog_source).
 index((path_segments_atom), 2, prolog_source, prolog_source).
+index((directory_source_files), 3, prolog_source, prolog_source).
 index((get_prolog_backtrace), 2, prolog_stack, prolog_stack).
 index((get_prolog_backtrace), 3, prolog_stack, prolog_stack).
 index((prolog_stack_frame_property), 2, prolog_stack, prolog_stack).
@@ -719,6 +726,11 @@ index((xml_quote_attribute), 2, sgml, sgml).
 index((xml_quote_cdata), 2, sgml, sgml).
 index((xml_name), 1, sgml, sgml).
 index((xml_name), 2, sgml, sgml).
+index((xml_basechar), 1, sgml, sgml).
+index((xml_ideographic), 1, sgml, sgml).
+index((xml_combining_char), 1, sgml, sgml).
+index((xml_digit), 1, sgml, sgml).
+index((xml_extender), 1, sgml, sgml).
 index((iri_xml_namespace), 2, sgml, sgml).
 index((iri_xml_namespace), 3, sgml, sgml).
 index((xml_is_dom), 1, sgml, sgml).
