@@ -19,7 +19,7 @@
 
     You should have received a copy of the GNU General Public
     License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
     As a special exception, if you link this library with other files,
     compiled with a Free Software compiler, to produce an executable, this
@@ -192,9 +192,12 @@ harmless_dcgexception(type_error(callable,_)).	% ex: phrase(27,L)
 contains_illegal_dcgnt(NT) :-
 	sub_term(I, NT),
 	nonvar(I),
-	( I = ! ; I = phrase(_,_,_) ), !.
-%	write(contains_illegal_nt(NT)),		% JW: we do not want to write
-%	nl.
+	illegal_dcgnt(I), !.
+
+illegal_dcgnt(!).
+illegal_dcgnt(phrase(_,_,_)).
+illegal_dcgnt((_->_)).
+
 
 		 /*******************************
 		 *	     ACTIVATE		*
