@@ -20,7 +20,7 @@
 
     You should have received a copy of the GNU Lesser General Public
     License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
     As a special exception, if you link this library with other files,
     compiled with a Free Software compiler, to produce an executable, this
@@ -37,6 +37,7 @@
 					% Basic output routines
 	    page//1,			% :Content
 	    page//2,			% :Head, :Body
+	    page//3,			% +Style, :Head, :Body
 	    html//1,			% :Content
 
 					% Option processing
@@ -71,7 +72,6 @@
 :- use_module(library(pairs)).
 :- use_module(library(sgml)).		% Quote output
 :- use_module(library(uri)).
-:- use_module(library(quintus)).	% for meta_predicate/1
 :- set_prolog_flag(generate_debug_info, false).
 
 :- meta_predicate
@@ -188,13 +188,13 @@ html_current_option(content_type(ContentType)) :-
 
 option_default(html_dialect, html).
 option_default(html_doctype,
-	       'HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" \
+	       'HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" \c
 	       "http://www.w3.org/TR/html4/loose.dtd"').
 option_default(xhtml_doctype,
-	       'html PUBLIC "-//W3C//DTD XHTML 1.0 \
-	       Transitional//EN" \
+	       'html PUBLIC "-//W3C//DTD XHTML 1.0 \c
+	       Transitional//EN" \c
 	       "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd"').
-option_default(html_content_type, 'text/html').
+option_default(html_content_type, 'text/html; charset=UTF-8').
 option_default(xhtml_content_type, 'application/xhtml+xml; charset=UTF-8').
 
 %%	init_options is det.
@@ -246,6 +246,7 @@ page(Head, Body) -->
 
 page(Style, Head, Body) -->
 	doctype,
+	content_type,
 	html_begin(html),
 	pagehead(Style, Head),
 	pagebody(Style, Body),
@@ -266,6 +267,14 @@ doctype -->
 doctype -->
 	[].
 
+content_type -->
+	{ html_current_option(content_type(Type))
+	}, !,
+	html_post(head, meta([ 'http-equiv'('content-type'),
+			       content(Type)
+			     ], [])).
+content_type -->
+	[].
 
 pagehead(_, Head) -->
 	{ functor(Head, head, _)
@@ -595,7 +604,7 @@ name(Name) -->
 	expand_attribute_value//1.
 
 attribute_value(List) -->
-	{ is_list(List) },
+	{ is_list(List) }, !,
 	attribute_value_m(List).
 attribute_value(Value) -->
 	attribute_value_s(Value).
@@ -630,7 +639,7 @@ search_parameters([H|T]) -->
 	search_parameter(H),
 	(   {T == []}
 	->  []
-	;   [&],
+	;   ['&amp;'],
 	    search_parameters(T)
 	).
 
@@ -1174,9 +1183,9 @@ meta_called([_|MT], [_|AT], Called, Tail) :- !,
 	html(html,?,?),
 	page(html,?,?),
 	page(html,html,?,?),
-	page(html,html,?,?),
-	pagehead(html,?,?),
-	pagebody(html,?,?),
+	page(+,html,html,?,?),
+	pagehead(+,html,?,?),
+	pagebody(+,html,?,?),
 	reply_html_page(html,html),
 	reply_html_page(+,html,html),
 	html_post(+,html,?,?).
@@ -1187,12 +1196,12 @@ meta_called([_|MT], [_|AT], Called, Tail) :- !,
 		 *******************************/
 
 :- multifile
-	emacs_prolog_colours:goal_colours/2,
-	emacs_prolog_colours:style/2,
-	emacs_prolog_colours:identify/2,
+	prolog_colour:goal_colours/2,
+	prolog_colour:style/2,
+	prolog_colour:message//1,
 	prolog:called_by/2.
 
-emacs_prolog_colours:goal_colours(Goal, Colours) :-
+prolog_colour:goal_colours(Goal, Colours) :-
 	html_meta_head(Goal, _Module, Head),
 	html_meta_colours(Head, Goal, Colours).
 
@@ -1291,27 +1300,27 @@ location_id(_, classify).
 :- op(990, xfx, :=).			% allow compiling without XPCE
 :- op(200, fy, @).
 
-emacs_prolog_colours:style(html(_), style(bold := @on,
-					  colour := magenta4)).
-emacs_prolog_colours:style(entity(_), style(colour := magenta4)).
-emacs_prolog_colours:style(html_attribute(_), style(colour := magenta4)).
-emacs_prolog_colours:style(html_xmlns(_), style(colour := magenta4)).
-emacs_prolog_colours:style(sgml_attr_function, style(colour := blue)).
-emacs_prolog_colours:style(http_location_for_id(_), style(bold := @on)).
-emacs_prolog_colours:style(http_no_location_for_id(_), style(colour := red, bold := @on)).
+prolog_colour:style(html(_),			[colour(magenta4), bold(true)]).
+prolog_colour:style(entity(_),			[colour(magenta4)]).
+prolog_colour:style(html_attribute(_),		[colour(magenta4)]).
+prolog_colour:style(html_xmlns(_),		[colour(magenta4)]).
+prolog_colour:style(sgml_attr_function,		[colour(blue)]).
+prolog_colour:style(http_location_for_id(_),	[bold(true)]).
+prolog_colour:style(http_no_location_for_id(_),	[colour(red), bold(true)]).
 
 
-emacs_prolog_colours:identify(html(Element), Summary) :-
-	format(string(Summary), '~w: SGML element', [Element]).
-emacs_prolog_colours:identify(entity(Entity), Summary) :-
-	format(string(Summary), '~w: SGML entity', [Entity]).
-emacs_prolog_colours:identify(html_attribute(Attr), Summary) :-
-	format(string(Summary), '~w: SGML attribute', [Attr]).
-emacs_prolog_colours:identify(sgml_attr_function, 'SGML Attribute function').
-emacs_prolog_colours:identify(http_location_for_id(Location), Summary) :-
-	format(string(Summary), 'ID resolves to ~w', [Location]).
-emacs_prolog_colours:identify(http_no_location_for_id(ID), Summary) :-
-	format(string(Summary), '~w: no such ID', [ID]).
+prolog_colour:message(html(Element)) -->
+	[ '~w: SGML element'-[Element] ].
+prolog_colour:message(entity(Entity)) -->
+	[ '~w: SGML entity'-[Entity] ].
+prolog_colour:message(html_attribute(Attr)) -->
+	[ '~w: SGML attribute'-[Attr] ].
+prolog_colour:message(sgml_attr_function) -->
+	[ 'SGML Attribute function'-[] ].
+prolog_colour:message(http_location_for_id(Location)) -->
+	[ 'ID resolves to ~w'-[Location] ].
+prolog_colour:message(http_no_location_for_id(ID)) -->
+	[ '~w: no such ID'-[ID] ].
 
 
 %	prolog:called_by(+Goal, -Called)
