@@ -19,7 +19,7 @@
 
     You should have received a copy of the GNU General Public
     License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
     As a special exception, if you link this library with other files,
     compiled with a Free Software compiler, to produce an executable, this
@@ -42,6 +42,7 @@
 	  ]).
 :- use_module(library(lists)).
 :- use_module(library(debug)).
+:- use_module(library(error)).
 :- use_module(library(memfile)).
 :- use_module(library(pairs)).
 :- use_module(library(option)).
@@ -196,6 +197,13 @@ list_item_prefix(ol, [w(N), '.', ' '|T], T) :-
 	atom_codes(N, [D]),
 	between(0'0, 0'9, D).
 
+%%	split_dt(+LineAfterDollar, -DT, -Rest)
+%
+%	First see whether the entire line is the item. This allows
+%	creating items holding : by using $ <tokens> :\n
+
+split_dt(In, DT, []) :-
+	append(DT, [':'], In), !.
 split_dt(In, DT, Rest) :-
 	append(DT, [':'|Rest0], In),
 	(   Rest0 == []
@@ -419,7 +427,8 @@ tag_name(w(Name), Tag, Order) :-
 
 rest_tag([], _, [], []) :- !.
 rest_tag(Lines, Indent, [], Lines) :-
-	Lines = [Indent-[@,w(_Name)|_]|_], !.
+	Lines = [Indent-[@,Word|_]|_],
+	tag_name(Word, _, _), !.
 rest_tag([L|Lines0], Indent, [L|VT], Lines) :-
 	rest_tag(Lines0, Indent, VT, Lines).
 
@@ -737,7 +746,10 @@ nv_pair(Option) -->
 	[ w(Name), =,'"'], tokens(Tokens), ['"'], !,
 	{ untag(Tokens, Atoms),
 	  atomic_list_concat(Atoms, Value0),
-	  catch(atom_number(Value0, Value), _, Value=Value0),
+	  (   atom_number(Value0, Value)
+	  ->  true
+	  ;   Value = Value0
+	  ),
 	  Option =.. [Name,Value]
 	}.
 
@@ -922,6 +934,7 @@ functor_name(String) :-
 	char_type(Char, lower).
 
 url_protocol(http).
+url_protocol(https).
 url_protocol(ftp).
 url_protocol(mailto).
 
@@ -1403,3 +1416,18 @@ token(Token) -->
 
 token(w(_)) :- !.
 token(Token) :- atom(Token).
+
+
+		 /*******************************
+		 *	     MESSAGES		*
+		 *******************************/
+
+:- multifile
+	prolog:message//1.
+
+prolog:message(pldoc(deprecated_tag(Name, Tag))) -->
+	[ 'PlDoc: Deprecated tag @~w (use @~w)'-[Name, Tag]
+	].
+prolog:message(pldoc(unknown_tag(Name))) -->
+	[ 'PlDoc: unknown tag @~w'-[Name]
+	].
