@@ -46,6 +46,7 @@ index((rdf_unload), 1, rdf_db, rdf_db).
 index((rdf_md5), 2, rdf_db, rdf_db).
 index((rdf_atom_md5), 3, rdf_db, rdf_db).
 index((rdf_graph_property), 2, rdf_db, rdf_db).
+index((rdf_set_graph), 2, rdf_db, rdf_db).
 index((rdf_graph), 1, rdf_db, rdf_db).
 index((rdf_source), 1, rdf_db, rdf_db).
 index((rdf_source), 2, rdf_db, rdf_db).
@@ -63,6 +64,9 @@ index((lang_matches), 2, rdf_db, rdf_db).
 index((rdf_current_ns), 2, rdf_db, rdf_db).
 index((rdf_register_ns), 2, rdf_db, rdf_db).
 index((rdf_register_ns), 3, rdf_db, rdf_db).
+index((rdf_current_prefix), 2, rdf_db, rdf_db).
+index((rdf_register_prefix), 2, rdf_db, rdf_db).
+index((rdf_register_prefix), 3, rdf_db, rdf_db).
 index((rdf_global_id), 2, rdf_db, rdf_db).
 index((rdf_global_object), 2, rdf_db, rdf_db).
 index((rdf_global_term), 2, rdf_db, rdf_db).
@@ -158,6 +162,7 @@ index((rdfs_find), 5, rdfs, rdfs).
 index((sparql_query), 3, sparql_client, sparql_client).
 index((sparql_set_server), 1, sparql_client, sparql_client).
 index((sparql_read_xml_result), 2, sparql_client, sparql_client).
+index((sparql_read_json_result), 2, sparql_client, sparql_client).
 index((turtle_name_start_char), 1, turtle_base, turtle_base).
 index((turtle_name), 1, turtle_base, turtle_base).
 index((turtle_read_name), 4, turtle_base, turtle_base).
