@@ -1,11 +1,9 @@
-/*  $Id$
-
-    Part of SWI-Prolog
+/*  Part of SWI-Prolog
 
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@cs.vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (C): 1985-2010, University of Amsterdam
+    Copyright (C): 1985-2012, University of Amsterdam
 			      VU University Amsterdam
 
     This program is free software; you can redistribute it and/or
@@ -81,6 +79,7 @@
 	    rdf_atom_md5/3,		% +Text, +Times, -MD5
 
 	    rdf_graph_property/2,	% ?Graph, ?Property
+	    rdf_set_graph/2,		% +Graph, +Property
 	    rdf_graph/1,		% ?Graph
 	    rdf_source/1,		% ?File
 	    rdf_source/2,		% ?DB, ?SourceURL
@@ -102,6 +101,9 @@
 	    rdf_current_ns/2,		% ?Alias, ?URI
 	    rdf_register_ns/2,		% +Alias, +URI
 	    rdf_register_ns/3,		% +Alias, +URI, +Options
+	    rdf_current_prefix/2,	% ?Alias, ?URI
+	    rdf_register_prefix/2,	% +Alias, +URI
+	    rdf_register_prefix/3,	% +Alias, +URI, +Options
 	    rdf_global_id/2,		% ?NS:Name, ?Global
 	    rdf_global_object/2,	% ?Object, ?NSExpandedObject
 	    rdf_global_term/2,		% Term, WithExpandedNS
@@ -196,10 +198,7 @@
 	ns/2,
 	rdf_meta_specification/3.	% UnboundHead, Module, Head
 :- dynamic
-	ns/2,			% ID, URL
-	rdf_source/5.		% DB, SourceURL, ModTimeAtLoad, Triples, MD5
-:- volatile
-	rdf_source/5.
+	ns/2.			% ID, URL
 :- discontiguous
 	term_expansion/2.
 
@@ -212,6 +211,16 @@
 		 *	     NAMESPACES		*
 		 *******************************/
 
+%%	rdf_current_prefix(?Alias, ?URI) is nondet.
+%%	rdf_register_prefix(?Alias, ?URI) is det.
+%%	rdf_register_prefix(?Alias, ?URI, +Options) is det.
+%
+%	Forward compatibility. Version 3 deprecates the _ns versions.
+
+rdf_current_prefix(Alias, URI)           :- rdf_current_ns(Alias, URI).
+rdf_register_prefix(Alias, URI, Options) :- rdf_register_ns(Alias, URI, Options).
+rdf_register_prefix(Alias, URI)          :- rdf_register_ns(Alias, URI).
+
 %%	rdf_current_ns(?Alias, ?URI) is nondet.
 %
 %	Query  predefined  namespaces  and    namespaces   defined  with
@@ -624,9 +633,11 @@ rdf_is_literal(literal(Value)) :-
 		 *	      SOURCE		*
 		 *******************************/
 
-%%	rdf_source_location(+Subject, -File:Line)
+%%	rdf_source_location(+Subject, -Location) is nondet.
 %
-%	Return the source-locations for triples for this subject.
+%	True when triples for Subject are loaded from Location.
+%
+%	@param Location is a term File:Line.
 
 rdf_source_location(Subject, Source) :-
 	findall(Source, rdf(Subject, _, _, Source), Sources),
@@ -645,7 +656,9 @@ rdf_source_location(Subject, Source) :-
 %	@param KeyValue	Term of the form Key(Value).
 
 rdf_statistics(sources(Count)) :-
-	predicate_property(rdf_source(_,_,_,_,_), number_of_clauses(Count)).
+	rdf_statistics_(graphs(Count)).
+rdf_statistics(graphs(Count)) :-		% version 3 compatibility
+	rdf_statistics_(graphs(Count)).
 rdf_statistics(subjects(Count)) :-
 	rdf_statistics_(subjects(Count)).
 rdf_statistics(properties(Count)) :-
@@ -675,6 +688,8 @@ rdf_statistics(triples_by_file(File, Count)) :-
 	    rdf_statistics_(triples(File, Count))
 	;   rdf_statistics_(triples(File, Count))
 	).
+rdf_statistics(triples_by_graph(Graph, Count)) :- % version 3 compatibility
+	rdf_statistics(triples_by_file(Graph, Count)).
 rdf_statistics(duplicates(Count)) :-
 	rdf_statistics_(duplicates(Count)).
 
@@ -867,24 +882,11 @@ check_loaded_cache(DB, Graphs, _) :-
 
 %%	rdf_load_db(+File) is det.
 %
-%	Load triples from a file created using rdf_save_db/2 and update
-%	the file administration.
+%	Load triples from a file created using rdf_save_db/2.
 
 rdf_load_db(File) :-
 	uri_file_name(URL, File),
-	rdf_load_db_no_admin(File, URL, Graphs),
-	(   (   is_list(Graphs)
-	    ->	member(DB, Graphs)
-	    ;	DB = Graphs
-	    ),
-	    rdf_md5(DB, MD5),
-	    rdf_statistics_(triples(DB, Triples)),
-	    rdf_graph_source_(DB, SourceURL, Modified),
-	    retractall(rdf_source(DB, _, _, _, _)),
-	    assert(rdf_source(DB, SourceURL, Modified, Triples, MD5)),
-	    fail
-	;   true
-	).
+	rdf_load_db_no_admin(File, URL, _Graphs).
 
 
 		 /*******************************
@@ -959,7 +961,6 @@ rdf_load(Spec, M:Options) :-
 	->  do_unload(Graph),
 	    catch(rdf_load_db_no_admin(CacheFile, cache(Graph), Graphs), _, fail),
 	    check_loaded_cache(Graph, Graphs, Modified),
-	    rdf_statistics_(triples(Graph, Triples)),
 	    Action = load
 	;   option(base_uri(BaseURI), Options, Graph),
 	    (	var(BaseURI)
@@ -979,11 +980,9 @@ rdf_load(Spec, M:Options) :-
 			 Cleanup),
 	    save_cache(Graph, SourceURL, Options),
 	    register_file_ns(NSList),
-	    rdf_statistics_(triples(Graph, Triples)),
-	    rdf_md5(Graph, MD5),
-	    assert(rdf_source(Graph, SourceURL, Modified, Triples, MD5)),
 	    format_action(Format, Action)
 	),
+	rdf_statistics_(triples(Graph, Triples)),
 	report_loaded(Action, SourceURL, Graph, Triples, T0, Options).
 
 format_action(triples, load) :- !.
@@ -1265,17 +1264,18 @@ rdf_storage_encoding('', plain).
 rdf_load_stream(xml, Stream, Options) :- !,
 	graph(Options, Graph),
 	rdf_transaction(process_rdf(Stream, assert_triples, Options),
-			parse(Graph)).
+			parse(Graph)),
+	rdf_graph_clear_modified_(Graph).
 rdf_load_stream(xhtml, Stream, M:Options) :- !,
 	graph(Options, Graph),
 	rdf_transaction(process_rdf(Stream, assert_triples,
 				    M:[embedded(true)|Options]),
-			parse(Graph)).
+			parse(Graph)),
+	rdf_graph_clear_modified_(Graph).
 rdf_load_stream(triples, Stream, Options) :- !,
 	graph(Options, Graph),
 	rdf_load_db_(Stream, Graph, _Graphs).
 
-
 %%	report_loaded(+Action, +Source, +DB, +Triples, +StartCPU, +Options)
 
 report_loaded(none, _, _, _, _, _) :- !.
@@ -1309,7 +1309,6 @@ rdf_unload(_).
 do_unload(DB) :-
 	rdf_transaction(rdf_retractall(_,_,_,DB),
 			unload(DB)),
-	retractall(rdf_source(DB, _, _, _, _)),
 	rdf_unset_graph_source(DB).
 
 
@@ -1371,6 +1370,9 @@ modified_graph(SourceURL, Graph) :-
 %
 %	    * hash(Hash)
 %	    Hash is the (MD5-)hash for the content of Graph.
+%	    * modified(Boolean)
+%	    True if the graph is modified since it was loaded or
+%	    rdf_set_graph/2 was called with modified(false).
 %	    * source(Source)
 %	    The graph is loaded from the Source (a URL)
 %	    * source_last_modified(?Time)
@@ -1385,6 +1387,8 @@ rdf_graph_property(Graph, Property) :-
 
 rdf_graph_property_(hash(Hash), Graph) :-
 	rdf_md5(Graph, Hash).
+rdf_graph_property_(modified(Boolean), Graph) :-
+	rdf_graph_modified_(Graph, Boolean, _).
 rdf_graph_property_(source(URL), Graph) :-
 	rdf_graph_source_(Graph, URL, _).
 rdf_graph_property_(source_last_modified(Time), Graph) :-
@@ -1393,6 +1397,17 @@ rdf_graph_property_(source_last_modified(Time), Graph) :-
 rdf_graph_property_(triples(Count), Graph) :-
 	rdf_statistics_(triples(Graph, Count)).
 
+%%	rdf_set_graph(+Graph, +Property) is det.
+%
+%	Set properties of Graph.  Defined properties are:
+%
+%	    * modified(false)
+%	    Set the modified state of Graph to false.
+
+rdf_set_graph(Graph, modified(Modified)) :-
+	must_be(oneof([false]), Modified),
+	rdf_graph_clear_modified_(Graph).
+
 
 %%	save_cache(+DB, +Cache) is det.
 %
@@ -1427,7 +1442,6 @@ assert_triples([H|_], _) :-
 %	statistics.
 
 rdf_reset_db :-
-	retractall(rdf_source(_,_,_,_,_)),
 	rdf_transaction(rdf_reset_db_, reset).
 
 
@@ -2147,7 +2161,13 @@ save_body_literal(lang(Lang, Value),
 save_body_literal(type(Type, DOM),
 		  NameText, _BaseURI, Out, Indent, Options) :-
 	rdf_equal(Type, rdf:'XMLLiteral'), !,
-	save_xml_literal(DOM, NameText, Out, Indent, Options).
+	(   atom(DOM)
+	->  format(Out, '~N~*|<', [Indent]),
+	    rdf_write_id(Out, NameText),
+	    format(Out, ' rdf:parseType="Literal">~w</', [DOM]),
+	    rdf_write_id(Out, NameText), write(Out, '>')
+	;   save_xml_literal(DOM, NameText, Out, Indent, Options)
+	).
 save_body_literal(type(Type, Value),
 		  NameText, BaseURI, Out, Indent, _) :- !,
 	format(Out, '~N~*|<', [Indent]),
