@@ -1,11 +1,9 @@
-/*  $Id$
-
-    Part of SWI-Prolog
+/*  Part of SWI-Prolog
 
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (C): 2007-2011, University of Amsterdam
+    Copyright (C): 2007-2012, University of Amsterdam
 			      VU University Amsterdam
 
     This program is free software; you can redistribute it and/or
@@ -20,7 +18,7 @@
 
     You should have received a copy of the GNU Lesser General Public
     License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
     As a special exception, if you link this library with other files,
     compiled with a Free Software compiler, to produce an executable, this
@@ -55,6 +53,21 @@
 :- use_module(library(thread)).
 :- use_module(library(apply)).
 
+:- predicate_options(rdf_list_library/2, 2,
+		     [ indent(atom),
+		       show_graph(boolean),
+		       show_source(boolean),
+		       show_virtual(boolean)
+		     ]).
+:- predicate_options(rdf_load_library/2, 2,
+		     [ concurrent(positive_integer),
+		       import(boolean),
+		       load(boolean),
+		       base_uri(atom),
+		       claimed_source(atom),
+		       not_found(oneof([error,warning,silent]))
+		     ]).
+
 /** <module> RDF Library Manager
 
 This module manages an ontology library. Such   a  library consists of a
@@ -74,7 +87,9 @@ The typical usage scenario is
 @author Jan Wielemaker
 */
 
-:- rdf_register_ns(lib, 'http://www.swi-prolog.org/rdf/library/').
+:- rdf_register_ns(lib,  'http://www.swi-prolog.org/rdf/library/').
+:- rdf_register_ns(void, 'http://rdfs.org/ns/void#').
+:- rdf_register_ns(vann, 'http://purl.org/vocab/vann/').
 
 :- dynamic
 	manifest/2,			% Path, Time
@@ -325,15 +340,16 @@ dry_load(Id, Level, Options) :-
 	->  merge_base_uri(Facets, Options, Options1),
 	    merge_source(Facets, Options1, Options2),
 	    merge_blanks(Facets, Options2, Options3),
+	    merge_format(Facets, Options3, Options4),
 	    (   \+ memberchk(virtual, Facets)
-	    ->  load_options(Options3, File, RdfOptions),
+	    ->  load_options(Options4, File, RdfOptions),
 		assert(command(Level, rdf_load(File, RdfOptions)))
 	    ;	assert(command(Level, virtual(File)))
 	    ),
 	    (	option(import(true), Options, true)
 	    ->	Level1 is Level + 1,
-	        forall(member(imports(_, Import), Facets),
-		       import(Import, Level1, Options3))
+		forall(member(imports(Type, Import), Facets),
+		       import(Import, Level1, [type(Type)|Options4]))
 	    ;	true
 	    )
 	;   existence_error(ontology, Id)
@@ -360,6 +376,13 @@ merge_blanks(Facets, Options0, Options) :-
 	;   Options = Options0
 	).
 
+merge_format(Facets, Options0, Options) :-
+	(   option(format(Format), Facets)
+	->  exclude(name_option(format), Options0, Options1),
+	    Options = [format(Format)|Options1]
+	;   Options = Options0
+	).
+
 name_option(Name, Term) :-
 	functor(Term, Name, 1).
 
@@ -380,9 +403,15 @@ load_option(Options, File, base_uri(BaseURI)) :-
 	atom_concat(Base0, File, BaseURI).
 load_option(Options, _File, blank_nodes(Share)) :-
 	option(blank_nodes(Share), Options).
+load_option(Options, _File, format(Format)) :-
+	option(format(Format), Options).
 
 %%	import(+URL, +Level, +Options) is det.
 
+import(Path, Level, Options) :-
+	option(type(data_dump), Options), !,
+	load_options(Options, Path, RdfOptions),
+	assert(command(Level, rdf_load(Path, RdfOptions))).
 import(Path, Level, Options) :-
 	(   (   library(Id, Path, _)
 	    ->	true
@@ -475,7 +504,8 @@ existing_url_source(url(http, URL)) :- !,
 
 rdf_list_library :-
 	rdf_update_library_index,
-	(   rdf_library_index(Id, title(Title)),
+	(   rdf_library_index(Id, title(TitleLiteral)),
+	    plain_string(TitleLiteral, Title),
 	    format('~w ~t~20|~w', [Id, Title]),
 	    (	rdf_library_index(Id, version(Version))
 	    ->	format(' (version ~w)', [Version])
@@ -486,6 +516,10 @@ rdf_list_library :-
 	;   true
 	).
 
+plain_string(String, String) :-
+	atomic(String), !.
+plain_string(lang(en, String), String) :- !.
+plain_string(lang(_, String), String) :- !.
 
 %%	rdf_library_index(?Id, ?Facet) is nondet.
 %
@@ -521,6 +555,11 @@ rdf_list_library :-
 %		Share is one of =share= or =noshare=.  Default is to
 %		share.
 %
+%		* format(Format)
+%		Format of the resource.  Can be used to overrule
+%		if the format as derived from the HTTP content type
+%		is wrong.
+%
 %		* provides_ns(URL)
 %		Ontology provides definitions in the namespace URL.
 %		The formal definition of this is troublesome, but in
@@ -602,8 +641,10 @@ attach_dir(Path, Visited) :-
 attach_dir(Path, Visited) :-
 	atom_concat(Path, '/*', Pattern),
 	expand_file_name(Pattern, Members),
-	(   member(Manifest, Members),
-	    is_manifest_file(Manifest)
+	(   manifest_file(MBase),
+	    rdf_extension(Ext),
+	    atomic_list_concat([Path, /, MBase, '.', Ext], Manifest),
+	    exists_file(Manifest)
 	->  process_manifest(Manifest)
 	;   print_message(silent, rdf(no_manifest(Path)))
 	),
@@ -627,12 +668,15 @@ hidden_base('cvs').			% Windows
 %	@param	Location is either a path name or a URL.
 
 process_manifest(Source) :-
-	(   uri_file_name(Source, Manifest0)
-	->  absolute_file_name(Manifest0, ManifestFile)
-	;   absolute_file_name(Source, ManifestFile)
-	),				% Manifest is a canonical filename
-	uri_file_name(Manifest, ManifestFile),
-	source_time(ManifestFile, MT),
+	(   web_url(Source)
+	->  uri_normalized(Source, Manifest)
+	;   uri_file_name(Source, Manifest0)
+	->  absolute_file_name(Manifest0, ManifestFile),
+	    uri_file_name(Manifest, ManifestFile)
+	;   absolute_file_name(Source, ManifestFile),
+	    uri_file_name(Manifest, ManifestFile)
+	),				% Manifest is a canonical URI
+	source_time(Manifest, MT),
 	(   manifest(Manifest, Time),
 	    (	MT =< Time
 	    ->  !
@@ -642,7 +686,7 @@ process_manifest(Source) :-
 		retractall(library_db(Id, URL, Facets)),
 		fail
 	    )
-	;   read_triples(ManifestFile, Triples),
+	;   read_triples(Manifest, Triples),
 	    process_triples(Manifest, Triples),
 	    print_message(informational, rdf(manifest(loaded, Manifest))),
 	    assert(manifest(Manifest, MT))
@@ -665,6 +709,9 @@ process_triples(Manifest, Triples) :-
 extract_namespace(Triples, Mnemonic, Namespace) :-
 	edge(Triples, Decl, lib:mnemonic, literal(Mnemonic)),
 	edge(Triples, Decl, lib:namespace, Namespace).
+extract_namespace(Triples, Mnemonic, Namespace) :-
+	edge(Triples, Decl, vann:preferredNamespacePrefix, literal(Mnemonic)),
+	edge(Triples, Decl, vann:preferredNamespaceUri, literal(Namespace)).
 
 %%	extract_ontology(+Triples, -Ontology) is nondet.
 %
@@ -675,13 +722,27 @@ extract_ontology(Triples, library(Name, URL, Options)) :-
 	(   ontology_type(Type)
 	->  file_base_name(URL, BaseName),
 	    file_name_extension(Name, _, BaseName),
-	    findall(Facet, facet(Triples, URL, Facet), Options)
+	    findall(Facet, facet(Triples, URL, Facet), Options0),
+	    sort(Options0, Options1),
+	    keep_specialized_facets(Options1, Options)
 	).
 
+keep_specialized_facets(All, Special) :-
+	exclude(more_general(All), All, Special).
+
+more_general(All, Facet) :-
+	generalized(Facet, Special),
+	memberchk(Special, All).
+
+generalized(imports(ontology, Path), imports(Other, Path)) :-
+	dif(Other, ontology).
+
 ontology_type(X) :-
 	(   rdf_equal(X, lib:'Ontology')
 	;   rdf_equal(X, lib:'Schema')
 	;   rdf_equal(X, lib:'Instances')
+	;   rdf_equal(X, void:'Dataset')
+	;   rdf_equal(X, void:'Linkset')
 	).
 
 %%	facet(+Triples, +File, -Facet) is nondet.
@@ -689,12 +750,8 @@ ontology_type(X) :-
 %	Enumerate facets about File from   Triples. Facets are described
 %	with rdf_library_index/2.
 
-facet(Triples, File, title(Title)) :-
-	edge(Triples, File, dc:title, literal(Title)).
-:- if(rdf_current_ns(dcterms, _)).
 facet(Triples, File, title(Title)) :-
 	edge(Triples, File, dcterms:title, literal(Title)).
-:- endif.
 facet(Triples, File, version(Version)) :-
 	edge(Triples, File, owl:versionInfo, literal(Version)).
 facet(Triples, File, comment(Comment)) :-
@@ -703,6 +760,8 @@ facet(Triples, File, base_uri(BaseURI)) :-
 	edge(Triples, File, lib:baseURI, BaseURI).
 facet(Triples, File, claimed_source(Source)) :-
 	edge(Triples, File, lib:source, Source).
+facet(Triples, File, format(Format)) :-
+	edge(Triples, File, lib:format, literal(Format)).
 facet(Triples, File, blank_nodes(Mode)) :-
 	edge(Triples, File, lib:blankNodes, literal(Mode)),
 	must_be(oneof([share,noshare]), Mode).
@@ -712,6 +771,10 @@ facet(Triples, File, imports(schema, Path)) :-
 	edge(Triples, File, lib:schema, Path).
 facet(Triples, File, imports(instances, Path)) :-
 	edge(Triples, File, lib:instances, Path).
+facet(Triples, File, imports(subset, Path)) :-
+	edge(Triples, File, void:subset, Path).
+facet(Triples, File, imports(data_dump, Path)) :-
+	edge(Triples, File, void:dataDump, Path).
 facet(Triples, File, provides_ns(NS)) :-
 	edge(Triples, File, lib:providesNamespace, NSDecl),
 	edge(Triples, NSDecl, lib:namespace, NS).
@@ -719,14 +782,39 @@ facet(Triples, File, uses_ns(NS)) :-
 	edge(Triples, File, lib:usesNamespace, NSDecl),
 	edge(Triples, NSDecl, lib:namespace, NS).
 facet(Triples, File, virtual) :-
-	edge(Triples, File, rdf:type, lib:'Virtual').
+	(   edge(Triples, File, rdf:type, lib:'Virtual')
+	;   edge(Triples, File, rdf:type, void:'Dataset')
+	;   edge(Triples, File, rdf:type, void:'Linkset')
+	) -> true.
 
 %%	edge(+Triples, ?S, ?P, ?O) is nondet.
 %
-%	Like rdf/3 over a list of Triples.
+%	Like rdf_has/3 over a list of Triples.
 
 edge(Triples, S, P, O) :-
-	member(rdf(S,P,O), Triples).
+	nonvar(P), !,
+	sub_p(SubP, P),
+	member(rdf(S,SubP,O), Triples).
+edge(Triples, S, P, O) :-
+	member(rdf(S,SubP,O), Triples),
+	sub_p(SubP, P).
+
+sub_p(P, P).
+sub_p(Sub, P) :-
+	(   nonvar(Sub)
+	->  sub_property_of(Sub, Sub1),
+	    sub_p(Sub1, P)
+	;   sub_property_of(Sub1, P),
+	    sub_p(Sub, Sub1)
+	).
+
+:- rdf_meta
+	sub_property_of(r,r).
+
+sub_property_of(void:subset,	     owl:imports).
+sub_property_of(dcterms:description, rdfs:comment).
+sub_property_of(void:dataDump,	     owl:imports).
+sub_property_of(dc:title,	     dcterms:title).
 
 %%	source_time(+Source, -Modified) is semidet.
 %
@@ -735,7 +823,7 @@ edge(Triples, S, P, O) :-
 %	@error	existence_error(Type, Source).
 
 source_time(URL, Modified) :-
-	sub_atom(URL, 0, _, _, 'http://'), !,
+	web_url(URL), !,
 	http_open(URL, Stream,
 		  [ header(last_modified, Date),
 		    method(head)
@@ -749,29 +837,34 @@ source_time(URL, Modified) :-
 source_time(File, Modified) :-
 	time_file(File, Modified).
 
+web_url(URL) :-
+	sub_atom(URL, 0, _, _, 'http://').
 
-%%	read_triples(+File, -Triples) is det.
+
+%%	read_triples(+URL, -Triples) is det.
 %
 %	Read RDF/XML or Turtle file into a list of triples.
 
-read_triples(File, Triples) :-
-	file_name_extension(_, rdf, File), !,
-	load_rdf(File, Triples).
-read_triples(File, Triples) :-
-	file_name_extension(_, ttl, File), !,
-	rdf_load_turtle(File, Triples, []).
+read_triples(FileURL, Triples) :-
+	uri_file_name(FileURL, File), !,
+	(   file_name_extension(_, rdf, File)
+	->  load_rdf(File, Triples)
+	;   rdf_load_turtle(File, Triples, [])
+	).
+read_triples(HTTPURL, Triples) :-
+	file_name_extension(_, Ext, HTTPURL),
+	setup_call_cleanup(
+	    http_open(HTTPURL, In, []),
+	    stream_triples(In, Ext, Triples),
+	    close(In)).
 
-%%	is_manifest_file(+Path) is semidet.
-%
-%	True if Path is the name of a manifest file.
+stream_triples(Stream, rdf, Triples) :-
+	load_rdf(stream(Stream), Triples).
+stream_triples(Stream, ttl, Triples) :-
+	rdf_load_turtle(stream(Stream), Triples, []).
 
-is_manifest_file(Path) :-
-	file_base_name(Path, File),
-	downcase_atom(File, Lwr),
-	file_name_extension(Base, Ext, Lwr),
-	manifest_file(Base),
-	rdf_extension(Ext), !.
 
+manifest_file('void').			% make order optional?
 manifest_file('Manifest').
 manifest_file('manifest').
 
